clear all
close all

% Example script for affine estimation of one block extracted from TO RF
% images

%%
% load two TO RF images
folder_name = ['frame' num2str(11) '/'];
load([folder_name 'rf2D.mat']);
im1 = rf2D;

folder_name = ['frame' num2str(12) '/'];
load([folder_name 'rf2D.mat']);
im2 = rf2D;

% compute analytic signals and extract their phases
[R1,R2]=ashahn(im1);
[S1,S2]=ashahn(im2);
P1_im1=atan2(imag(R1),real(R1));
P2_im1=atan2(imag(R2),real(R2));
P1_im2=atan2(imag(S1),real(S1));
P2_im2=atan2(imag(S2),real(S2));

%%
% normalized spatial (carrier) frequencies in axial and lateral diections
f = [0.1 0.16];

% central point of the block
P = [3181 60];

% block size
L=[128 16];

% initial displacement estimated with pixelwise correlation
d_init = [33 1];

%%
% display the images and the block

% on the first frame
figure
imagesc(sqrt(abs(hilbert(im1)))); colormap gray
hold
rectangle('Position', [P(2)-L(2)/2 P(1)-L(1)/2 L(2) L(1)],'EdgeColor','w')

% on the second frame with the initial displacement considered
figure
imagesc(sqrt(abs(hilbert(im2)))); colormap gray
hold
rectangle('Position', [P(2)-L(2)/2+d_init(2) P(1)-L(1)/2+d_init(1) L(2) L(1)]...
    ,'EdgeColor','w')

%%
% estimate the affine transform between the two blocks
[U V] = PhaseAdjustAffine...
    (P1_im1(P(1)-ceil(L(1)/2)+1:P(1)-ceil(L(1)/2)+L(1),...
    P(2)-ceil(L(2)/2)+1:P(2)-ceil(L(2)/2)+L(2)),...
    P2_im1(P(1)-ceil(L(1)/2)+1:P(1)-ceil(L(1)/2)+L(1),...
    P(2)-ceil(L(2)/2)+1:P(2)-ceil(L(2)/2)+L(2))...
    ,P1_im2(P(1)-ceil(L(1)/2)+1+d_init(1):P(1)-ceil(L(1)/2)+L(1)+d_init(1),...
    P(2)-ceil(L(2)/2)+1+d_init(2):P(2)-ceil(L(2)/2)+L(2)+d_init(2))...
    ,P2_im2(P(1)-ceil(L(1)/2)+1+d_init(1):P(1)-ceil(L(1)/2)+L(1)+d_init(1),...
    P(2)-ceil(L(2)/2)+1+d_init(2):P(2)-ceil(L(2)/2)+L(2)+d_init(2)),...
    f(2),f(1),'rectwin')

V(1)=V(1)+d_init(1);
U(1)=U(1)+d_init(2);

%%
% compute the displacement of the 4 corners of the block with the estimated
% affine transform

d_ul(1) = V(2)*(-L(2)/2) + V(3)*(-L(1)/2)+V(1);
d_ul(2) = U(2)*(-L(2)/2) + U(3)*(-L(1)/2)+U(1);

d_ur(1) = V(2)*(L(2)/2) + V(3)*(-L(1)/2)+V(1);
d_ur(2) = U(2)*(L(2)/2) + U(3)*(-L(1)/2)+U(1);

d_dl(1) = V(2)*(-L(2)/2) + V(3)*(L(1)/2)+V(1);
d_dl(2) = U(2)*(-L(2)/2) + U(3)*(L(1)/2)+U(1);

d_dr(1) = V(2)*(L(2)/2) + V(3)*(L(1)/2)+V(1);
d_dr(2) = U(2)*(L(2)/2) + U(3)*(L(1)/2)+U(1);

%%
% plot the deformed block on the target image
figure
imagesc(sqrt(abs(hilbert(im2)))); colormap gray
hold
line([P(2)-L(2)/2+d_ul(2) P(2)+L(2)/2+d_ur(2)], [P(1)-L(1)/2+d_ur(1) P(1)-L(1)/2+d_ul(1)]...
    ,'Color','w')
line([P(2)+L(2)/2+d_ur(2) P(2)+L(2)/2+d_dl(2)], [P(1)-L(1)/2+d_ur(1) P(1)+L(1)/2+d_dl(1)]...
    ,'Color','w')
line([P(2)+L(2)/2+d_dl(2) P(2)-L(2)/2+d_dr(2)], [P(1)+L(1)/2+d_dl(1) P(1)+L(1)/2+d_dr(1)]...
    ,'Color','w')
line([P(2)-L(2)/2+d_dl(2) P(2)-L(2)/2+d_ul(2)], [P(1)+L(1)/2+d_dl(1) P(1)-L(1)/2+d_ul(1)]...
    ,'Color','w')



